// Code for Friend:
  // Written By: Chris Kang, David Kalman, Rick Schutte, and Misha Gliwny
  // March 8, 2020
  // Completed as Part of Course Work for "Useless Machine" Project in ARC385 at the Daniels Faculty in the University of Toronto


// Include all the Needed Libraries for this Project

#include <Stepper.h>  // Stepper Library
#include <SPI.h>  // Serial Peripheral Interface Communication Library for RFID
#include <MFRC522.h>  // Library for Communication between Arduino and RFID
#include "LedControl.h" // LED Control Library
#include "binary.h" // LED Control Library

// Global Setup for Communication with Second Arduino

char mystr[1] = "0"; //String Array for Communicating Between Both Boards


// Global Setup for Stepper Motor Functions

Stepper myStepper(stepsPerRevolution, 5, 7, 6, 8);  // // initialize the stepper library on pins 8 through 11
const int stepsPerRevolution = 2048;        // The Number of Steps Per revolution for the Stepper Motor
const int RevolutionsPerMinute = 12;        // Adjustable range of 28BYJ-48 stepper is 0~17 rpm


// Global Setup for RFID Functions

#define RST_PIN   9 // Reset Pin
#define SS_PIN    10  // Communication Pin

  // Initializing MFRC Communication

  MFRC522 Mymfrc522(SS_PIN, RST_PIN);   // Create MFRC522 instance
  

// Global Setup for Dot Matrix Functions

LedControl lc = LedControl(2, 4, 3, 1); // Set up LedControl(DIN,CLK,CS,1);

  // Library of Faces :)

  byte pf_0[8] = {B00100100, B01000010, B00000000, B00100100, B00000000, B00100100, B00011000, B00000000};  // Happy, friend is full of Joy 
  byte pf_1[8] = {B00100100, B01000010, B00000000, B00100100, B00000000, B00000000, B00011000, B00000000};  // Neutral, they don't hate you but Friend is unsure
  byte pf_2[8] = {B00100100, B01000010, B00000000, B00100100, B00000000, B00100100, B00000000, B00000000};  // Stone Cold and Emotionless

  int count = 0;
  byte lastPerson[4] = {0, 0, 0, 0};

  // Read Faces Subroutine

void drawImage(byte* myImage)
{
  for (int i = 0; i < 8; i++) lc.setRow(0, i, myImage[i]);
}


void setup()
{
  // RFID Setup
  Serial.begin(9600); // Open Serial Communication and Set the Rate to 9600
  SPI.begin();         // Initialize SPI bus
  Mymfrc522.PCD_Init();  // Initialize MFRC522 Card
  
  // Stepper Setup
  myStepper.setSpeed(RevolutionsPerMinute); // Set Speed of the Stepper Motors

  // Dot Matrix Setup
  lc.shutdown(0, false);  // Set Brightness of Dot Matrix to a Medium Value
  lc.setIntensity(0, 1);  // Clear the Display
  lc.clearDisplay(0); // Itialize Clear Display
}


void loop()
{
  // Look for new cards, and select one if present
  if ( ! Mymfrc522.PICC_IsNewCardPresent() || ! Mymfrc522.PICC_ReadCardSerial() ) {
    delay(50);
    return;
  }

  // Now a card is selected. The UID and SAK is in mfrc522.uid.
  if ((Mymfrc522.uid.uidByte[0] == lastPerson[0]) &&
      (Mymfrc522.uid.uidByte[1] == lastPerson[1]) &&
      (Mymfrc522.uid.uidByte[2] == lastPerson[2]) &&
      (Mymfrc522.uid.uidByte[3] == lastPerson[3])) {
    if (count < 11) count++;
  }
  else if (count == 11) {
    count = 0;
  }
  else {
    count = 0;
    lastPerson[0] = Mymfrc522.uid.uidByte[0];
    lastPerson[1] = Mymfrc522.uid.uidByte[1];
    lastPerson[2] = Mymfrc522.uid.uidByte[2];
    lastPerson[3] = Mymfrc522.uid.uidByte[3];
  }

// This Switch Function Contains the Different Stages of Friend Becoing Your Friend

  switch (count) {
    case 0:
      drawImage(pf_1);  // Draw Face One
      mystr[1] = '0'; // Prepare a "0" to send to the Second Arduino
      Serial.println(mystr);  // Print the Vlaue Above (For Debugging)
      Serial.write(mystr, 1); // Send a "0" to the Other Arduino to Turn on the LEDs
      myStepper.step(-stepsPerRevolution); // Run Motor Backwards 1 Revolution
      break;
    case 1:
      drawImage(pf_1);
      mystr[1] = '1';
      Serial.println(mystr);
      Serial.write(mystr, 1); 
      myStepper.step(-stepsPerRevolution); // Run Motor Backwards 1 Revolution
      break;
    case 2:
      drawImage(pf_1);
      mystr[1] = '2';
      Serial.println(mystr);
      Serial.write(mystr, 1);
      myStepper.step(-stepsPerRevolution / 2); // Run Motor Backwards 1/2 Revolution
      break;
    case 3:
      drawImage(pf_2);
      mystr[1] = '3';
      Serial.println(mystr);
      Serial.write(mystr, 1);
      delay(1000);
      break;
    case 4:
      drawImage(pf_2);
      mystr[1] = '4';
      Serial.println(mystr);
      Serial.write(mystr, 1); 
      myStepper.step(-stepsPerRevolution / 4); // Run Motor Backwards 1/4 Revolution
      break;
    case 5:
      drawImage(pf_2);
      mystr[1] = '5';
      Serial.println(mystr);
      Serial.write(mystr, 1);
      myStepper.step(stepsPerRevolution / 4); // Run Motor Forewards 1/4 Revolution
      myStepper.step(-stepsPerRevolution / 2);  // Run Motor Backwards 1/2 Revolution
      break;
    case 6:
      drawImage(pf_0);
      mystr[1] = '6';
      Serial.println(mystr);
      Serial.write(mystr, 1);
      myStepper.step(stepsPerRevolution / 2); // Run Motor Forewards 1/2 Revolution
      myStepper.step(-stepsPerRevolution / 2); // Run Motor Backwards 1/2 Revolution
      break;
    case 7:
      drawImage(pf_1);
      mystr[1] = '7';
      Serial.println(mystr);
      Serial.write(mystr, 1);
      myStepper.step(stepsPerRevolution / 8); // Run Motor Forewards 1/8 Revolution
      delay(500);
      myStepper.step(-stepsPerRevolution / 16); // Run Motor Backwards 1/16 Revolution
      delay(500);
      myStepper.step(stepsPerRevolution / 4); // Run Motor Forewards 1/4 Revolution
      break;
    case 8:
      drawImage(pf_0);
      mystr[1] = '8';
      Serial.println(mystr);
      Serial.write(mystr, 1);
      myStepper.step(stepsPerRevolution / 2); // Run Motor Forewards 1/2 Revolution
      break;
    case 9:
      drawImage(pf_0);
      mystr[1] = '9';
      Serial.println(mystr);
      Serial.write(mystr, 1);
      myStepper.step(stepsPerRevolution); // Run Motor Forewards 1 Revolution
      break;

      // Congratulations!! You have made a New Friend!!

  }

  // Clear Motor Pins in Preparation to Make a New Friend

  digitalWrite(5, 0);
  digitalWrite(6, 0);
  digitalWrite(7, 0);
  digitalWrite(8, 0);
  delay(1000);
}
